/***
*
* Utility library mainly used by objects/unFrontEnd/DeleteSubApplication.xml
*
*/


/**
	Simple class holding the number of devices belonging to an application
 **/
class SubApplicationDevices{
	public SubApplicationDevices( string applicationName, int numDevices){
		this._numDevices = numDevices;
		this._applicationName = applicationName;
	}
	
	public string getApplicationName(){
		return this._applicationName;
	}
	
	public int getNumDevices(){
		return this._numDevices;
	}	
	
	int _numDevices;
	string _applicationName;
	
};


/**
Given the DP of a frontend, returns an vector of SubApplicationDevices containing the
name of each application and their number of devices
**/
vector<SubApplicationDevices> unFrontEnd_DeleteSubApp_getAppAndNumDevices( string sFrontEndDp){
	string sFrontEndDpNormalized = dpSubStr(sFrontEndDp, DPSUB_SYS_DP);
	if (dpExists(sFrontEndDpNormalized)==false){
		DebugTN(__FUNCTION__, "[ERROR]", "FrontEnd Dp does not exist", sFrontEndDp);
		return makeVector();
	}
	
	string sFrontEndName = dpGetAlias(sFrontEndDpNormalized+".");
	
	//1-get front end application
	vector<string> vListSubApp = unFrontEnd_DeleteSubApp_getFrontEndApplications(sFrontEndDpNormalized);
		
	
	//2-for each app, get number of devices
	vector<SubApplicationDevices> results;
	for(int i=0; i<vListSubApp.count(); i++){
		if (vListSubApp[i]=="COMMUNICATION"){
			continue;
		}
		dyn_string dsDevices;
		unGenericDpFunctions_getPlcDevices( dpSubStr( sFrontEndDp, DPSUB_SYS),
																		sFrontEndName,
																		vListSubApp[i],
																		"*",
																		"*",
																		dsDevices);
		SubApplicationDevices sad = SubApplicationDevices( vListSubApp[i], dynlen(dsDevices));
		results.append( sad);
	}
	
	//3-build vector and return it
	return results;
}

/**
 Given the DP of a frontend, simply returns its list of sub applications
**/
vector<string> unFrontEnd_DeleteSubApp_getFrontEndApplications( string sFrontEndDp){
	string sFrontEndDpNormalized = dpSubStr(sFrontEndDp, DPSUB_SYS_DP);
	if (dpExists(sFrontEndDpNormalized)==false){
		DebugTN(__FUNCTION__, "[ERROR]", "FrontEnd Dp does not exist", sFrontEndDp);
		return makeVector();
	}
	dyn_string dsListSubApp;
	if( dpGet(sFrontEndDpNormalized+".configuration.subApplications", dsListSubApp)!=0){
		DebugTN(__FUNCTION__, "[ERROR]", getLastError());
		return makeVector();
	}
	vector<string> results = dsListSubApp;

	return results;
}

/**
Given the DP of a front end and a specific application name,
the function removes the subapplication from the front end conbfiguration DP
returns 1 if successful, and negative value if an error occurs.
**/
int unFrontEnd_DeleteSubApp_remove( string sFrontEndDp, string sApplicationName){
	string sFrontEndDpNormalized = dpSubStr(sFrontEndDp, DPSUB_SYS_DP);
	if (dpExists(sFrontEndDpNormalized)==false){
		DebugTN(__FUNCTION__, "[ERROR]", "FrontEnd Dp does not exist", sFrontEndDp);
		return -1;
	}
	
	if ( sApplicationName == "COMMUNICATION"){
		DebugTN(__FUNCTION__, "[WARNING]", "SubApplication COMMUNICATION should not be removed (dont know why though...)");
		return 1;
	}
	
	dyn_string dsDevices;
	unGenericDpFunctions_getPlcDevices( dpSubStr( sFrontEndDp, DPSUB_SYS),
																	dpGetAlias(dpSubStr( sFrontEndDp, DPSUB_SYS)+"."),
																	sApplicationName,
																	"*",
																	"*",
																	dsDevices);
	if(dynlen(dsDevices)!=0){
		DebugTN(__FUNCTION__, "[ERROR}", "SubApplication "+sApplicationName+" can not be deleted. It has "+dynlen(dsDevices)+" devices attached.");
		return -3;
	}
	
	dyn_string dsSubApp, dsImportTime, dsBool, dsAnalog, dsEvent;
	if (dpGet(sFrontEndDpNormalized+".configuration.subApplications", dsSubApp,
				 sFrontEndDpNormalized+".configuration.importTime", dsImportTime,
				 sFrontEndDpNormalized+".configuration.archive_bool", dsBool,
				 sFrontEndDpNormalized+".configuration.archive_analog", dsAnalog,
				 sFrontEndDpNormalized+".configuration.archive_event", dsEvent)
		 != 0)
	{
		DebugTN(__FUNCTION__, "[ERROR]", getLastError());
		return -1;
	}
		
	int iIndexApp = dsSubApp.indexOf(sApplicationName);
	if( iIndexApp<0){
		DebugTN(__FUNCTION__, "[WARNING]", "SubApplication "+sApplicationName+" can not be deleted (not found in list of subApplications of "+sFrontEndDp );
		return 1;
	}	
	
	dsSubApp.removeAt(iIndexApp);
	dsImportTime.removeAt(iIndexApp);
	dsBool.removeAt(iIndexApp);
	dsAnalog.removeAt(iIndexApp);
	dsEvent.removeAt(iIndexApp);
	
	if (dpSetWait(sFrontEndDpNormalized+".configuration.subApplications", dsSubApp,
				 sFrontEndDpNormalized+".configuration.importTime", dsImportTime,
				 sFrontEndDpNormalized+".configuration.archive_bool", dsBool,
				 sFrontEndDpNormalized+".configuration.archive_analog", dsAnalog,
				 sFrontEndDpNormalized+".configuration.archive_event", dsEvent)
		 != 0)
	{
		DebugTN(__FUNCTION__, "[ERROR]", getLastError());
		return -2;
	}
		
	return 1;
}
